/*
        MMC5603 Library
        Author : Seonguk Jeong
        Created at : 2023-12-27
        Updated at : 2023-12-27
        Target : STM32
*/
#pragma once
#ifndef ROVERDYN_STM32_LIBRARY_SENSORS_MMC5603_H_
#define ROVERDYN_STM32_LIBRARY_SENSORS_MMC5603_H_

#include <stdio.h>
#include <stdint.h>
#include <String.h>
#include <Configure.h>

// Register Definition
#define MMC5603_XOUT0                   0x00
#define MMC5603_XOUT1                   0x01
#define MMC5603_YOUT0                   0x02
#define MMC5603_YOUT1                   0x03
#define MMC5603_ZOUT0                   0x04
#define MMC5603_ZOUT1                   0x05
#define MMC5603_XOUT2                   0x06
#define MMC5603_YOUT2                   0x07
#define MMC5603_ZOUT2                   0x08
#define MMC5603_TOUT                    0x09
#define MMC5603_STATUS1                 0x18
#define MMC5603_ODR                     0x1A
#define MMC5603_INTERNAL_CONTROL0       0x1B
#define MMC5603_INTERNAL_CONTROL1       0x1C
#define MMC5603_INTERNAL_CONTROL2       0x1D
#define MMC5603_ST_X_TH                 0x1E
#define MMC5603_ST_Y_TH                 0x1F
#define MMC5603_ST_Z_TH                 0x20
#define MMC5603_ST_X                    0x27
#define MMC5603_ST_Y                    0x28
#define MMC5603_ST_Z                    0x29
#define MMC5603_PRODUCT_ID              0x39

// MMC5603 Settings
#define MMC5603_ODR_75HZ                0x00
#define MMC5603_ODR_150HZ               0x01
#define MMC5603_ODR_255HZ               0x02
#define MMC5603_ODR_1000HZ              0x03
#define MMC5603_BANDWIDTH00             0x00
#define MMC5603_BANDWIDTH01             0x01
#define MMC5603_BANDWIDTH10             0x02
#define MMC5603_BANDWIDTH11             0x03
#define MMC5603_MODE_SINGLE             0x00
#define MMC5603_MODE_CONTINUOUS         0x01
#define MMC5603_NULL_OUTPUT             524288

// Predeifinition
#define MMC5603_ADDR                    0x30


struct MMC5603_CONFIG{
	I2C_HandleTypeDef i2c;
	uint8_t addr;
	uint32_t timeout = 10000;
	uint8_t odr;
	bool continuous;
	uint8_t bandwidth;
};

class MMC5603{
private:
	MMC5603_CONFIG  config;
public:
	void Init(MMC5603_CONFIG mmc5603_config);
	uint8_t getDeviceId();
	int32_t *getRawMag();
	bool isReady();
};

#endif /* ROVERDYN_STM32_LIBRARY_SENSORS_MMC5603_H_ */
